/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.listeners;

import java.io.File;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;

import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.util.FileNamePatternUtil;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.component.framework.util.FileNamePatternUtil.RegexFileNameFilter;

/**
 * A polling util used to schedule recursive task to be executed. Extend this
 * abstract class and implement the run method that will be launched each N ms.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class DirectoryPoller extends TimerTask {

    /**
     * The timer used for task scheduling
     */
    protected Timer timer;

    /**
     * The polling period
     */
    protected long period;

    /**
     * The first task execution delay
     */
    protected long delay;

    private File directory;

    private FileTransferExternalListener listener;

    private RegexFileNameFilter fileFilter;

    /**
     * Call processFile on the external listener, at repeated time, for each
     * file located in the specified directory
     * 
     * @param delay
     * @param period
     * @param directory
     * @param listener
     */
    public DirectoryPoller(long delay, long period, File directory, String pattern,
            FileTransferExternalListener listener) {
        this.delay = delay;
        this.period = period;
        this.timer = new Timer();
        this.directory = directory;
        this.listener = listener;
        if (StringHelper.isNullOrEmpty(pattern)) {
            fileFilter = FileNamePatternUtil.getInstance().buildFileNameFilterFromWildChar("*");
        } else {
            fileFilter = FileNamePatternUtil.getInstance().buildFileNameFilterFromWildChar(pattern);
        }
    }

    /**
     * Start the task processing
     * 
     */
    public void startProcessing() {
        timer.schedule(this, delay, period);
    }

    /**
     * Stop the task processing
     * 
     */
    public void stopProcessing() {
        timer.cancel();
    }

    @Override
    public void run() {
        try {
            if (directory.canRead()) {
                File[] list = directory.listFiles(fileFilter);

                if (list.length > 0) {
                    for (File file : list) {
                        if (FileTransferUtils.checkAccess(file)) {
                            try {
                                listener.processFile(file);
                                // Remember to release the lock
                            } catch (IOException e) {
                                listener.getLogger().log(Level.SEVERE,
                                        "Can't process file " + file.getName(), e);
                            }
                        } else {
                            listener.getLogger().warning(
                                    file.getName() + " is locked, don't process it");
                        }
                    }
                }
            } else {
                this.listener.getLogger().log(Level.INFO,
                        "Failed to read polled directory : [" + directory.getAbsolutePath() + "]");
            }
        } catch (Exception e) {
            if (!(e instanceof InterruptedException)) {
                listener.getLogger().log(Level.WARNING, e.getMessage(), e);
            }
        }
    }
}
