/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.listeners;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.logging.Level;

import javax.jbi.messaging.MessagingException;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.CharEncoding;
import org.ow2.petals.bc.filetransfer.FileTransferComponent;
import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.component.framework.util.StringHelper;

import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_TRANSFER_MODE_ATTACHMENT;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_TRANSFER_MODE_CONTENT;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.POLLING_PERIOD;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.TRANSFER_MODE;

/**
 * Listen to a directory
 * 
 * @author alouis
 * 
 */
public class FileTransferExternalListener extends AbstractExternalListener {

    private DirectoryPoller poller;

    private File workspace;

    /**
     * Return the workspace directory
     * 
     * @return
     */
    public File getWorkspaceDirectory() {
        if (this.workspace == null) {
            this.workspace = new File(this.getComponent().getContext().getWorkspaceRoot());
        }
        return this.workspace;
    }

    @Override
    public void start() throws PEtALSCDKException {

        // retrieve the polling period
        final String s_polling = this.getExtensions().get(POLLING_PERIOD);
        final long period = Long.parseLong(s_polling);
        this.getLogger().finest("set polling period to :" + period + "ms");

        // retrieve file name pattern
        final String pattern = this.getExtensions().get(FileTransferConstants.FILE_PATTERN);
        // start the poller
        try {
            final File workingDirectory = FileTransferUtils.getReadDirectory(this.getExtensions());
            this.poller = new DirectoryPoller(0, period, workingDirectory, pattern, this);
            this.poller.startProcessing();
            this.getLogger().info("start polling directory : " + workingDirectory);
        } catch (final IOException e) {
            throw new PEtALSCDKException(e);
        }

    }

    @Override
    public void stop() throws PEtALSCDKException {
        try {
            this.getLogger().info(
                    "stop polling directory : "
                            + FileTransferUtils.getReadDirectory(this.getExtensions()));
        } catch (final IOException e) {
            // do nothing
        }
        this.poller.stopProcessing();
    }

    /**
     * Send the content of the file to the JBI bus, then remove it
     * 
     * @param file
     * @throws IOException
     */
    public void processFile(File file) throws IOException {
        this.getLogger().info("process file : " + file.getName());

        // move file in a work or backup dir before processing
        File processedFile = null;
        final File backupDirectory = FileTransferUtils.getBackupDirectory(this.getExtensions());
        if (backupDirectory == null) {
            processedFile = this.moveFile(file, new File(System.getProperty("java.io.tmpdir")));
        } else {
            processedFile = this.moveFile(file, backupDirectory);
        }

        //
        // process files
        //

        // transfer mode
        String transferMode = this.getExtensions().get(TRANSFER_MODE);
        this.getLogger().finest("transfer mode : " + transferMode);
        if (StringHelper.isNullOrEmpty(transferMode)) {
            transferMode = ENUM_TRANSFER_MODE_CONTENT;
            this.getLogger().finest("using default mode : " + transferMode);
        }

        Exchange exchange = null;
        try {
            if (ENUM_TRANSFER_MODE_CONTENT.equalsIgnoreCase(transferMode)) {
                exchange = this.copySource(processedFile);
            } else if (ENUM_TRANSFER_MODE_ATTACHMENT.equalsIgnoreCase(transferMode)) {
                exchange = this.copyAttachment(processedFile, file.getName());
            } else {
                throw new PEtALSCDKException("Transfer mode not recognized :" + transferMode);
            }
            if (backupDirectory == null) {
                ((FileTransferComponent) this.getComponent()).addTransferedFile(exchange,
                        processedFile);
            }
        } catch (final Exception e) {
            this.getLogger().log(Level.SEVERE, e.getMessage(), e);
        }

    }

    /**
     * Send the file as a Source
     * 
     * @param file
     * @throws MessagingException
     * @throws FileNotFoundException
     *             file can not be read
     * @throws PEtALSCDKException
     */
    private Exchange copySource(File file) throws MessagingException, FileNotFoundException,
            PEtALSCDKException {
        // create an exchange set with the ServiceUnit values
        final Exchange exchange = this.createConsumeExchange(this.getConsumes(),
                MEPConstants.IN_ONLY_PATTERN);
        final StreamSource source = new StreamSource(new FileInputStream(file));
        exchange.setInMessageContent(source);
        this.send(exchange);

        return exchange;
    }

    /**
     * Send the file as an attachment
     * 
     * @param file
     * @throws MessagingException
     * @throws PEtALSCDKException
     */
    private Exchange copyAttachment(File file, String fileName) throws MessagingException,
            PEtALSCDKException {
        // create an exchange set with the ServiceUnit values
        final Exchange exchange = this.createConsumeExchange(this.getConsumes(),
                MEPConstants.IN_ONLY_PATTERN);
        try {
            FileTransferUtils.addFileAsAttachment(file, fileName, exchange.getInMessage());
        } catch (IOException e) {
            new PEtALSCDKException(e);
        }
        final String textString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><attached-files><file-name>"
                + fileName + "</file-name></attached-files>";
        try {
            exchange.setInMessageContent(new StreamSource(new ByteArrayInputStream(textString
                    .getBytes(CharEncoding.UTF_8))));
        } catch (UnsupportedEncodingException e) {
            // not possible!
        }

        this.send(exchange);

        return exchange;
    }

    /**
     * Move the file to the waste directory
     * 
     * @param file
     *            original file to process
     * @param destDir
     *            working file directory
     * @return the new file reference
     * @throws IOException
     *             fail to move the file
     */
    private File moveFile(File file, File destDir) throws IOException {
        this.getLogger().finest(
                "Move file from " + file.getAbsolutePath() + " to " + destDir.getAbsolutePath());
        final String target = "backup-" + FileTransferUtils.addDateToFileName(file.getName());
        final File newFile = new File(destDir, target);
        final boolean success = file.renameTo(newFile);
        if (!success) {
            // if the renameTO method is processed between 2 different File
            // System, the Java method may return false. In this case we try to
            // copy the file and remove the old one.
            FileUtils.copyFile(file, newFile);
            file.delete();
        }
        return newFile;
    }

}
