package org.ow2.petals.bc.filetransfer.service;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/***
 * Check if the specified file exist(Only absolute path are allowed). Request exemple : <q0:checkFile
 * xmlns:q0="http://petals.ow2.org/components/filetransfer/version-2">
 * <q0:filepath>/mypath/myfile.xml</q0:filepath> </q0:checkFile>
 * 
 * @author "Mathieu CARROLLE - PetalsLink"
 * 
 */
public class CheckFileService extends AbstractGetOperationService {

    public CheckFileService(ConfigurationExtensions extensions, Logger logger) {
        super(extensions, logger);
    }

    @Override
    public void process(Exchange exchange) {
        boolean exist = false;
        if (this.logger.isLoggable(Level.INFO)) {
            this.logger.info("JBI exchange receive - process checkFile operation");
        }
        try {
            if (!exchange.isInOutPattern()) {
                final StringBuilder errorSB = new StringBuilder();
                errorSB.append("Exchange pattern for the checkFile operation must be ");
                errorSB.append(Message.MEPConstants.IN_OUT_PATTERN.value());
                throw new JBIException(errorSB.toString());
            }
            try {
                final String filepath = this.extractFilePath(exchange);
                File file = new File(filepath);
                exist = file.exists();
                exchange.getOutMessage().setContent(
                        UtilFactory.getSourceUtil().createSource(
                                this.generateResponse(exist, exchange, filepath)));
                if (this.logger.isLoggable(Level.FINEST)) {
                    this.logger.log(Level.FINEST, "file path : " + filepath);
                }
            } catch (FileNotFoundException e) {
                this.logger.log(Level.WARNING, e.getMessage());
                final StringBuilder builder = new StringBuilder();
                builder.append(e.getMessage());
                this.setIOFaultOnExchange(exchange, builder.toString());
            } catch (MessagingException e) {
                this.logger.log(Level.WARNING, e.getMessage());
                final StringBuilder builder = new StringBuilder();
                builder.append(e.getMessage());
                this.setIOFaultOnExchange(exchange, builder.toString());
            } catch (PEtALSCDKException e) {
                this.logger.log(Level.WARNING, e.getMessage());
                final StringBuilder builder = new StringBuilder();
                builder.append(e.getMessage());
                this.setIOFaultOnExchange(exchange, builder.toString());
            }
        } catch (Exception e) {
            this.logger.log(Level.WARNING, e.getClass().getSimpleName() + " : " + e.getMessage());
            exchange.setError(new Exception(e));
        }
    }

    /**
     * Exctract the xml filepath element in the incomming jbi message
     * @param exchange
     * @return content of the filepath element
     * @throws MessagingException
     * @throws FileNotFoundException
     */
    private String extractFilePath(Exchange exchange) throws MessagingException,
            FileNotFoundException {
        Document doc = exchange.getInMessageContentAsDocument();
        Node rootNode = doc.getFirstChild();
        Node pattern = null;
        if (rootNode != null) {
            pattern = XMLUtil.findChild(rootNode, "filepath", null, false);
        }
        if (pattern == null) {
            throw new FileNotFoundException("Can't find filepath node");
        }
        return pattern.getTextContent();
    }

    /**
     * Generate the jbi response message
     * @param checkFile
     * @param exchange
     * @param filepath
     * @return
     */
    private String generateResponse(Boolean checkFile, Exchange exchange, String filepath) {
        StringBuilder builder = new StringBuilder();
        builder.append("<tns:" + exchange.getOperationName() + "Response xmlns:tns='");
        builder.append(FileTransferConstants.FILETRANSFER_SERVICE_NS);
        builder.append("'>");
        builder.append("<tns:filepath>");
        builder.append(filepath);
        builder.append("</tns:filepath>");
        builder.append("<tns:exist>");
        builder.append(checkFile ? "TRUE" : "FALSE");
        builder.append("</tns:exist>");
        builder.append("</tns:" + exchange.getOperationName() + "Response>");
        return builder.toString();
    }

}
