/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.service;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.lang.CharEncoding;
import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;

/**
 * The file transfer BC JBI listener. Writes a file to hard disk on incoming JBI
 * message.
 * 
 * @version
 * @since Petals 2.0
 * @author Adrien LOUIS - EBMWebSourcing
 * 
 */

public class GetFilesService extends AbstractGetOperationService {

    public GetFilesService(final ConfigurationExtensions extensions, final Logger logger) {
        super(extensions, logger);
    }

    /**
     * Process the exchange : get the files in the configured directory, <br>
     * using filepattern set in the exchange <br>
     * files are returned as attachment in the OUT response <br>
     * OUT message contains files names <br>
     * the file is move/deleted after read
     */
    @Override
    public void process(final Exchange exchange) {
        this.logger.log(Level.INFO, "JBI Exchange received - getFiles service requested");
        if (exchange.isInOnlyPattern() || exchange.isRobustInOnlyPattern()) {
            exchange
            .setError(new JBIException(
                    "the exchange pattern is not compatible. Operation getFiles is InOut or InOptionalOut."));
        } else {
            try {
                final String filePattern = this.extractFilePattern(exchange);
                final File readDirectory = FileTransferUtils.getReadDirectory(this.extensions);
                this.logger.log(Level.INFO, "read files from : " + readDirectory.getAbsolutePath());
                this.logger.log(Level.INFO, "file pattern : " + filePattern);
                final File[] files = this.listFiles(readDirectory, filePattern);
                final NormalizedMessage out = exchange.getOutMessage();
                final List<String> fileNameList = new ArrayList<String>();
                for (final File file : files) {
                    this.logger.info("file found : " + file.getName());
                    // check if file can be read
                    if (!FileTransferUtils.checkAccess(file)) {
                        this.logger.warning(file.getName() + " is locked, don't process it");
                    }
                    // move file in a work or backup dir before processing
                    final File processedFile = FileTransferUtils.moveFile(file, this.extensions);
                    // copy file as attachment
                    FileTransferUtils.addFileAsAttachment(processedFile, file.getName(), out);
                    fileNameList.add(file.getName());
                }
                // generate OUT response report
                final String report = generateFileNameList(fileNameList);
                exchange.setOutMessageContent(new StreamSource(new ByteArrayInputStream(report
                        .getBytes(CharEncoding.UTF_8))));
            } catch (final MessagingException e) {
                exchange.setError(new Exception(e));
                this.logger.log(Level.SEVERE, e.getMessage(), e);
            } catch (final IOException e) {
                try {
                    this.logger.warning("can not retrieve file(s). " + e.getClass().getName() + " : "
                            + e.getMessage());
                    this.setIOFaultOnExchange(exchange, e.getMessage());
                } catch (final JBIException e1) {
                    this.logger.log(Level.SEVERE, e1.getMessage(), e1);
                    exchange.setError(e1);
                }
            }
            this.logger.log(Level.INFO, "getFiles service is correctly terminated");
        }
    }

    /**
     * generates the XML representation of the file name list
     * 
     * @param names
     * @param namespace
     * @return
     */
    private static String generateFileNameList(final List<String> names) {
        final StringBuffer result = new StringBuffer();
        result.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        result.append("<tns:getFilesResponse xmlns:tns='");
        result.append(FileTransferConstants.FILETRANSFER_SERVICE_NS);
        result.append("'>");
        for (final String string : names) {
            result.append("<tns:filename>");
            result.append(string);
            result.append("</tns:filename>");
        }
        result.append("</tns:getFilesResponse>");
        return result.toString();
    }
}
