/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.filetransfer.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.nio.channels.OverlappingFileLockException;
import java.util.ArrayList;
import java.util.List;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.util.StringHelper;

import static org.ow2.petals.bc.filetransfer.FileTransferConstants.BACKUP_DIRECTORY;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.READ_DIRECTORY;

/**
 * @author Roland NAUDIN - EBM WebSourcing
 * 
 */
public class FileTransferUtils {

    /**
     * Evaluate a file path to resolve environment variables (place holders).
     */
    public static final String evaluateFilePath(String filePath) {
        String result = filePath;
        List<String> keys = extractPlaceHolderKeys(filePath);
        for (String key : keys) {
            result = filePath.replace("${" + key + "}", System.getenv(key));
        }
        // update file separator according to the OS dependent separator
        if (File.separator.equals("/")) {
            result = result.replace("\\", "/");
        } else {
            result = result.replace("/", "\\");
        }
        return result;
    }

    /**
     * Return the working directory
     * 
     * @return
     * @throws PEtALSCFException
     *             no working directory specified in the SU
     */
    public static final File getReadDirectory(ConfigurationExtensions extensions)
            throws IOException {
        String directory = extensions.get(READ_DIRECTORY);
        if (StringHelper.isNullOrEmpty(directory)) {
            throw new FileNotFoundException("'" + READ_DIRECTORY + "' parameter is not set");
        }
        directory = FileTransferUtils.evaluateFilePath(directory);
        File working = new File(directory);
        if (!working.exists()) {
            throw new FileNotFoundException("'" + READ_DIRECTORY + "' parameter does not exist :"
                    + directory);
        }
        return working;
    }

    /**
     * Return the backup directory, {@code null} if it is not configured.
     * 
     * @return the backup directory
     */
    public static final File getBackupDirectory(ConfigurationExtensions extensions)
            throws IOException {
        File working = null;
        String directory = extensions.get(BACKUP_DIRECTORY);
        if (!StringHelper.isNullOrEmpty(directory)) {
            directory = FileTransferUtils.evaluateFilePath(directory);
            working = new File(directory);
            if (!working.exists()) {
                throw new FileNotFoundException(BACKUP_DIRECTORY + " does not exist :" + directory);
            }
        }
        return working;
    }

    /**
     * Send the file as a Source
     * 
     * @param file
     * @param message
     * @throws IOException
     * @throws MessagingException
     */
    public static final void copyFileToContent(File file, NormalizedMessage message)
            throws IOException, MessagingException {
        StreamSource source = new StreamSource(new FileInputStream(file));
        message.setContent(source);
    }

    /**
     * Send the file as an attachment
     * 
     * @param file
     * @throws MessagingException
     * @throws IOException
     */
    public static final void addFileAsAttachment(File file, String filename,
            NormalizedMessage message) throws MessagingException, IOException {
        DataHandler attachment = new DataHandler(file.toURI().toURL());
        message.addAttachment(filename, attachment);
    }

    /**
     * move file to the backup directory specified in the SU<br>
     * if no backup directory if set, move to the Temp directory. <br>
     * The file need to be moved before being read
     * 
     * @param file
     * @return the moved file
     * @throws IOException
     */
    public static final File moveFile(File file, ConfigurationExtensions extensions)
            throws IOException {
        // move file in a work or backup dir before processing
        File processedFile = null;
        File backupDirectory = getBackupDirectory(extensions);
        if (backupDirectory == null) {
            File tmpDirectory = new File(System.getProperty("java.io.tmpdir"));
            processedFile = new File(tmpDirectory, "petals-bc-filetransfer-" + file.getName()
                    + System.nanoTime());
        } else {
            String target = "backup-" + addDateToFileName(file.getName());
            processedFile = new File(backupDirectory, target);
        }
        if (!file.renameTo(processedFile)) {
            throw new IOException("Can not move file " + file.getName() + " to "
                    + processedFile.getAbsolutePath());
        }
        return processedFile;
    }

    /**
     * Add the current time as suffix of the filename
     * 
     * @param filename
     * @return
     */
    public static final String addDateToFileName(String filename) {
        final String result;
        final long time = System.nanoTime();
        // Format the string to contains always 14 characters.
        final String sTime = String.format("%1$014d", time);
        int index = filename.lastIndexOf('.');
        if (index > 0) {
            String prefix = filename.substring(0, filename.lastIndexOf('.'));
            String suffix = filename.substring(filename.lastIndexOf('.'), filename.length());
            result = prefix + "-" + sTime + suffix;
        } else {
            result = filename + "-" + sTime;
        }

        return result;
    }

    /**
     * check if the file can be read
     * 
     * @param file
     * @return
     */
    public static final boolean checkAccess(File file) {
        boolean result = false;
        FileChannel channel = null;
        FileLock lock = null;
        try {
            // try to access the file to test if it can be read
            channel = new RandomAccessFile(file, "rw").getChannel();
            // Use the file channel to create a lock on the file.
            // This method blocks until it can retrieve the lock.
            lock = channel.lock();
            lock.release();
            result = true;
        } catch (OverlappingFileLockException e) {
            // do nothing
        } catch (IOException e) {
            // do nothing
        } finally {
            if (channel != null) {
                try {
                    channel.close();
                } catch (IOException e1) {
                    // do nothing
                }

            }
        }
        return result;
    }

    /**
     * Extract the place holders.
     * 
     * @param entryValue
     * @return List of place holders
     */
    private static final List<String> extractPlaceHolderKeys(String entryValue) {
        List<String> result = new ArrayList<String>();
        String[] keys = entryValue.split("[${]");
        for (String string : keys) {
            int index = string.indexOf("}");
            if (index > 0) {
                result.add(string.substring(0, index));
            }
        }
        return result;
    }
}
